pro mk_fem, year, week, noaoslos=noaoslos, dirout=dirout, asca=asca, version=version, interval_aoslos=interval_aoslos
;
;+
;NAME:
;	mk_fem
;PURPOSE:
;	Generate a FEM file
;INPUT:
;	year	- year number (eg: 92)
;	week	- week number
;OPTIONAL KEYWORD INPUT:
;	noaoslos - if set, do not run DSNFIL (AOSLOS) program - the file already
;		  exists.  If it creates it, it will delete it automatically.
;	dirout	- output directory for FEM file. If not defined, it goes into
;		  $DIR_GEN_FEM.  It is used for temporary regeneration of
;		  FEM files - not disturb on-line FEM files.
;	asca	- If set, then read the ASCA spacecraft orbital solutions and
;		  write the reults to the DIR_ASCA_FEM directory
;       interval_aoslos - interval (seconds) for aoslos file generation
;                         (default = mk_fem_aoslos defalut = 15 seconds)
;
;RESTRICTION:
;	CAUTION: The file IDs (FIDs) are derived from the first minute of sun
;	as calculated on the ground MINUS FIVE MINUTES.  This is to insure
;	that a spacecraft day is not split between two files.  As such, the
;	date and week # of a FID might not match that of the FEM data.
;HISTORY:
;	Written Dec-91 by M.Morrison
;	28-Feb-92 (MDM) - changed from MK_FID to MK_FEM
;			  cleaned up the code since it no longer was using
;			  random AOSLOS files that potentially overlapped in time
;	17-Mar-92 (MDM) - Updated FEM structure to handle the SIRIUS path
;			  name.
;	26-Apr-92 (MDM) - Regenerated all files with no time difference between
;			  the predicted first minute of sun and the FileID.
;			  Previous values had the FileID 5 minutes before the
;			  the predicted first minute of sun.
;	26-May-92 (MDM) - Made MK_FEM_WRT always use /clobber
;	27-Jul-92 (MDM) - corrected bug which was eliminating station contacts
;			  from the FEM structure when the night/day transition
;			  occurred when an SAA or station contact was happening
;	 7-Apr-93 (MDM) - Added /ASCA option
;V3.0	21-Feb-94 (MDM) - Increased the resolution of the day/night boundary
;			  by incorporating a call to GET_SUNSET
;			  (increase is from 15 sec to 1 sec resolution)
;	16-Mar-94 (MDM) - Change to the way GET_SUNSET is called since the
;			  new FEM file does not exist yet, and GET_SUNSET was
;			  doing a RD_FEM within it.
;V3.1	13-Apr-94 (MDM) - Corrected so that ASCA calculations do not use
;			  GET_SUNSET 
;V4.0	23-Mar-95 (MDM) - Updated to handle Wallops ground station.
;			- Had to modify the FEM structure so a new extension
;			  was used.  Eliminated the antenna fields.  Expanded
;			  the possible number of station contacts within a
;			  single orbit from 3 to 6
;			- Added /VERSION keyword
;       03-jan-00       - y2k fix
;V5.0    5-Dec-2002 (SLF) - explicitly derive ORBIT PERIOD from aoslosfile
;                           and pass to mk_fem_wrt.
;V6.0    8-Dec-2002 (SLF) - change to mk_fem_file.pro to increment 
;                           orbits/file to 16 if ORBIT_PERIOD falls below 96 min
;                           (happend in Yohkoh last months circa 01_37)
;V7.0    9-Dec-2002 (SLF) - add INTERVAL keyord (->mk_fem_aoslos)
;                           add mk_fem version to FEMDATA.SPARE(0,*)
;                           add INTERVAL       to FEMDATA.SPARE(1,*)
;-
;
progver = 7.0
if not keyword_set(interval_aoslos) then interval_aoslos=15    ; AOSLOS Interval
if (keyword_set(version)) then begin
    year = progver
    return
end
;
qfirst = 0
if (keyword_set(asca)) then begin
    if (n_elements(dirout) eq 0) then dirout = '$DIR_ASCA_FEM'
    if ((year eq 93) and (week eq 8)) then qfirst = 1
end else begin
    if (n_elements(dirout) eq 0) then dirout = '$DIR_GEN_FEM'
    if ((year eq 91) and (week eq 35)) then qfirst = 1
end
;
if (not keyword_set(noaoslos)) then begin
    mk_fem_aoslos, year, week, aoslos_file=aoslos_file, asca=asca, $
       interval=interval_aoslos
    orbit_period=yoh_aoslos2orbperiod(aoslos_file)
    box_message,'Orbit period derived from aoslos file: ' + strtrim(orbit_period)
end else begin
    ;ff = findfile('/ydb/ref_scratch/*.ascii')
;    outdir00 = concat_dir(getenv('ydb'), 'ref_scratch')
outdir00 = '/db_isass5/ydb/ref_scratch'
    ff = file_list(outdir00, '*.ascii')
    aoslos_file = ff(0)		    ;TODO - get the aoslos filename in a better manner
end
openr, lun, aoslos_file, /get_lun
;
if (qfirst) then begin
    rev = 1
end else begin
    last_week = week - 1
    last_year = year
    if (last_week eq 0) then begin
	last_week = 53
	last_year = year - 1
        IF (last_year LT 0) THEN last_year = 100 + last_year ;y2k fix.
    end
    rd_fem, last_week, last_year, last_fem_data, indir=dirout, asca=asca
    rev = max(last_fem_data.sc_rev) + 1
end
;
rev_day = 1
last_day_week = 99
last_day_time = intarr(7)
last_ds79 = 0
last_msod = 0
day = 4000
time = 0
path = 0

fem_struct, fem_data=fem_data
;
antenna = strarr(2) + '*'
station = strarr(2) + '*'
st_station = [0,0]
en_station = [0,0]
qdump = 0
qdebug = 0
istation = 0
qin_contact = [0,0]
qmiddle = [0,0]
qeof = 0
;margin = 5*60.*1000	;back off 5 minutes
margin = 0.		;changed margin to zero for .02 generation (26-Apr-92)
qfirst = 1	;looking for first day transition - start storing information after that
irec = 0L
while (not qeof) do begin

    rd_orbit_event, lun, orbit, start_rec=irec, end_rec=irec, eof=qeof, /new, /plus
    orbit = orbit(0)
    if (irec eq 0) then lorbit = orbit
  
    timarr_ut = orbit.timarr
    ex2int, timarr_ut, msod, ds79
    del = long((ds79 - day)*86400. + (msod-time)/1000.)		;time in sec from s/c day

    qfirst_contact = 0		;used to figure out how to log the antenna - only log changes
    for ii=0,1 do begin
        qavoid = 0
        str_temp0 = strtrim(lorbit.station(ii), 2)
        str_temp1 = strtrim(orbit.station(ii), 2)
        if ((str_temp1 eq '+') and (str_temp0 eq '')) then qavoid = 1
        if ((str_temp1 eq '') and (str_temp0 eq '+')) then qavoid = 1

        if (qdebug) then print, irec, ' -- ', fmt_tim(timarr_ut), $
					'qfirst, qdump, qmiddle', qfirst, qdump, qmiddle, ' SCD:', orbit.sc_day

	if ((orbit.station(ii) ne lorbit.station(ii)) and (not qavoid) and (not qfirst)) then begin
            if ((str_temp1 eq '+') or (str_temp1 eq '')) then begin		;end of contact
	        if (qdebug) then print, 'End of station contact'
	        en_station = del
                qin_contact(ii) = 0
	        qmiddle(ii) = 0

	        fem_data.st_station(istation) = st_station(ii)
	        fem_data.en_station(istation) = en_station
	        fem_data.st$station(istation) = byte(station(ii))
	        ;;fem_data.st$antenna(istation,0) = byte(antenna(ii))
	        istation = istation + 1
	        if (antenna(ii) eq '*') then istation = istation-1	;special case where a station contact starts before the
							;first day and ends after first day - it gets logged in last
							;day of previous week
            endif else begin
	        if (qdebug) then print, 'Start of station contact'
	        st_station(ii) = del
	        antenna(ii) = orbit.antenna(ii)
	        station(ii) = strtrim(orbit.station(ii),2)	;could be first or second character!
                qin_contact(ii) = 1
	        qmiddle(ii) = 1
                qfirst_contact = 1
            endelse
	end
    end

;;    ;---- Look for changes in antenna 
;;    if ((qin_contact) and (not qfirst_contact)) then begin
;;        if (orbit.antenna ne lorbit.antenna) then begin
;;	    ;antenna change in the middle of a contatct
;;	    if (qdebug) then print, 'Change of antenna in middle of contact'
;;	    fem_data.st$antenna(istation,1) = byte(orbit.antenna)
;;	    fem_data.cng_antenna(istation) = del
;;        end
;;    endif

    if (orbit.saa ne lorbit.saa) then begin
	if (orbit.saa eq '?') then begin		;saa started
	    if (qdebug) then print, 'Start of SAA
	    fem_data.st_saa = del
	    qmiddle = [1,1]		;in the middle of saa or contact extraction
	end else begin
	    if (qdebug) then print, 'End of SAA
	    fem_data.en_saa = del
	    qmiddle = [0,0]
	end
    end

    if (orbit.sc_day ne lorbit.sc_day) then begin	;transition into day or night
        if (orbit.sc_day ne '*') then begin		;start of day
	    if (qdebug) then print, 'S/C Day ', fmt_tim(timarr_ut)
	    qdump = 0
	    if (not qfirst) then qdump = 1		;write out the buffered information (if not in middle of saa or station)
							;have to wait for end of day to write out information

	    next_time = msod		;buffer to recover after dump previous day
	    next_day  = ds79
	    next_path = fix(strmid(orbit.path, 6, 4))

	    if (qfirst) then begin
		time = msod
		day  = ds79
		path = fix(strmid(orbit.path, 6, 4))
	    end
	    qfirst = 0
	    ;;istation = 0	;reset station contact counter - correct??	;removed 27-Jul-92 (MDM)
        end else begin					;start of night (print out line)
	    if (qdebug) then print, 'S/C Night ', fmt_tim(timarr_ut)
	    night = del
        end
    end

    if (qdump and (not total(qmiddle))) then begin
	int2ex, time, day, tarr
	if (week eq ex2week(tarr)) then begin		;inside of the desired week
	
	    if (qdebug) then print, 'Writting record to file

	    fem_data.time = time			;true s/c day start time
	    fem_data.day  = day
	    fem_data.path = path
            int2ex, time, day, day_time

            ;----- Generate the fileid
	    fday = day
            ftime = time - margin                        ;have the definition of the day period actually back off by "margin"
            check_time, ftime, fday 
            int2ex, ftime, fday, timarr
            fileID = string(timarr(6), timarr(5), timarr(4), '.', timarr(0), timarr(1), FORMAT = '(3i2.2,a,2i2.2)')

	    if (not qfirst) then begin
                if (day_time(4) ne last_day_time(4)) then begin	;New day
                    rev_day = 1
                end
	    end

	    ;---- MDM added 21-Feb-94
	    daytim = anytim2ints(fem_data)
	    nighttim0 = anytim2ints(daytim, off=night)
	    ;;nighttim = get_sunset(nighttim0, nighttim0)	;MDM removed 16-Mar-94
	    if (keyword_set(asca)) then nighttim = nighttim0 $	;MDM added conditional 13-Apr-94
				else nighttim = get_sunset(fem_night=nighttim0)
	    night = int2secarr(nighttim, daytim)
	    ;---- MDM finished adding

            fem_data.night  = night

            fem_data.st$fileid = byte(fileid)
            fem_data.week   = week
            fem_data.year   = day_time(6)
            fem_data.iday   = ex2dow(day_time)
            fem_data.day_rev= rev_day
            fem_data.sc_rev = rev

	    ;---- Use the adjusted sc day times (5 minutes for FID margin) when deciding which file to put the entry in

            fem_data.spare(0,*)=progver
            fem_data.spare(1,*)=interval_aoslos
	    mk_fem_wrt, fem_data, /clobber, dirout=dirout, asca=asca, use_orbit_period=orbit_period

	    last_day_time = day_time
	    rev = rev + 1
            rev_day = rev_day + 1
	end

	istation = 0				;reset index for station
	fem_struct, fem_data = fem_data	;re-zero the data fields

	time = next_time
	day  = next_day
	path = next_path

	qdump = 0
    end

    lorbit = orbit
    last_msod = msod
    last_ds79 = ds79
    irec = irec + 1
end
;
if (not keyword_set(noaoslos)) then begin
    cmd = 'rm -f ' + aoslos_file
    spawn, cmd
end

free_lun, lun
end 

