pro run_dsnfil, starttime, path_out=path_out, orbit_file=orbit_file,	$
			   file_out=file_out, verbose=verbose, ASCA=ASCA,$
			   fversion=version
;+
;  Name:
;    run_dsnfil
;  Purpose:
;    Driver to run the fortran routine DSNFIL. 
;
;  Calling Example:
;    run_dsnfil,'92 1 4 0 0 0 60 1000',file_out=file_out,path='/ydb'
;
;  Inputs:
;    starttime	= Starting time in the AOSLOS format.  This can be a
;		  string or a 8 element vector.  
;                 YY MM DD HH MM SS Interval Number
;                 Interval is in integer seconds.
;		  Number is the number of steps.
;		  YY can be 19xx or just xx (e.g., 1992 or 92).
;
;  Optional Input Keywords:
;   orbit_file	= The file name containing the orbital state vector.
;		  If not supplied, run_dsnfil will choose the file
;		  which is closest to the mid-time in the requested
;		  calculation.
;   path_out	= The output file path.  If not supplied, it will use
;		  cd,current=path_out to get the current directory.
;   ASCA	= If set, use the ASCA orbital solutions.
;
;  Optional Output Keywords:
;   file_out	= The output file name.
;   fversion	= The version number of the FORTRAN dsnfil (AOSLOS) program.
;  
;  Restrictions:
;    Only set up for Ultrix, OSF, and SGI systems.
;
;    The executables (dsnfil_dec, dsnfil_alpha, and dsnfil_sgi) must reside 
;    somewhere in the IDL path described by !path.
;
;  Modification History:
;    Written, 19 Feb 1992, F.-M. Chou, and J. R. Lemen
;    Modified, 6 Mar 1992, J. R. Lemen, Enabled dsnfil_dec or dsnfil_sgi option
;    Modified, 5 Sep 1992, JRL, Handle the case of path_out='' correctly
;    Modified,15 Mar 1993, JRL, Delete output file first in case of different owner.
;     7-Apr-93, JRL,  Added the ASCA keyword (reads ASCA orbital solutions)
;    16-Mar-94, MDM,  Modified to use FILE_LIST instead of FINDFILE because of
;		      too many files in the directory problem on the SGI.
;    21-Mar-94, HSH,  Corrected error introduced 16-Mar-94 so that user can
;		      specify the orbital file directly again
;    12-Dec-94, JRL,  Suppressed informational messages unless /verbose
;    27-Feb-95, GAL,LS  Updated fortran exec. for DSNFIL for ALPHAs.
;     1-Mar-95, MDM,  Corrected syntax of 27-Feb modification
;                     Turned the problem code into a case statement
;    15-mar-95, JRL   Optionally return the version of DSNFIL (AOSLOS).
;		      Updated some of the comments.
;    20-mar-95, JRL   Minor change to a diagnostic message
;     3-jan-00, JRL   Y2K fixes applied
;    12-dec-02, JRL   Change the format of the command to allow more than
;                          99999 intervals (to allow 1s time resolution)
;-
on_error,2                        ;Return to caller if an error occurs

; Set up the rigidity file name:

if keyword_set(ASCA) then rigid_file_path = '$DIR_ASCA_ORBIT_SOL/rigidity.d'  $
		     else rigid_file_path = '$DIR_GEN_ORBIT_SOL/rigidity.d'

rigid_file = findfile(rigid_file_path)
rigid_file = rigid_file(0)
if strlen(rigid_file) eq 0 then begin
  print,' *** Error in run_dsnfil  ***'
  print,'     Could not find/open the rigidity file =',rigid_file_path
  ans = '' & read,'* Enter <cr> to continue ',ans
  return
endif


;  Set up the executable name:

dsnfil_exec = 'dsnfil'			; Base name

case strtrim(strlowcase(!version.os),2) of
    'ultrix': dsnfil_exec = dsnfil_exec+'_dec'
    'osf':    dsnfil_exec = dsnfil_exec+'_alpha'
    'irix':   dsnfil_exec = dsnfil_exec+'_sgi'
    else: begin
	  print,'****  Error in run_dsnfil   ****'
	  print,'      Unsupported operation system =',!version.os
	  print,'      Must be IRIX (SGI), OSF (ALPHA) or ULTRIX (DEC)'
	  ans = '' & read,'* Enter <cr> to continue ',ans
	  return
	end
endcase
;
dsnfil_path = path_lib(dsnfil_exec,/nopro)		; Get the path name
dsnfil_path = dsnfil_path(0)
if strlen(dsnfil_path) eq 0 then begin
  print,' *** Error in run_dsnfil  ***'
  print,"     Can't find the executable = ",dsnfil_exec
  ans = '' & read,'* Enter <cr> to continue ',ans
  return
end

if n_elements(starttime) eq 1 then begin		; Must be a string
   time_str = strtrim(starttime,2)
   while strpos(time_str,'  ') ne -1 do begin	       ; Eliminate double blanks
      i = strpos(time_str,'  ')
      time_str = strmid(time_str,0,i)+strmid(time_str,i+1,strlen(time_str))
   endwhile
   time_arr = long(str2arr(time_str,delim=" "))
endif else time_arr = starttime

if n_elements(time_arr) ne 8 then begin
  print,' *** Error in run_dsnfil *** '
  print,'     Incorrect start time format'
  print,'     Check the following: ',starttime
  ans = '' & read,'* Enter <cr> to continue ',ans
  return
endif

if time_arr(0) lt 1950 then begin
  if time_arr(0) le 49 then time_arr(0) = time_arr(0) + 2000 else $
                            time_arr(0) = time_arr(0) + 1900
endif

start_arr = [time_arr(3:5),0L,time_arr([2,1,0])]	; External format

; Check to see if output path is supplied:

if n_elements(path_out) eq 0 then cd,current=path_out 
if path_out eq '' then cd,current=path_out

; Check to see if the orbit file is supplied:

if n_elements(orbit_file) eq 0 then begin
    if keyword_set(ASCA) then sea_dir = '$DIR_ASCA_ORBIT_SOL' $
                         else sea_dir = '$DIR_GEN_ORBIT_SOL'  
    files = file_list(sea_dir, 'orbit_*')	;MDM modified 16-Mar-94
    ;;findfile(sea_files)
end else begin
    files = findfile(orbit_file)	; User specified his own case
end


n_files = n_elements(files)				; Number of files
orbit_times = lonarr(7,n_files)				; External format
for i=0,n_files-1 do begin
    openr,lun,files(i),/get_lun
    buf = '' & readf,lun,buf & buf = strmid(buf,0,12)
    orbit_times(6,i) = long(strmid(buf,0,2)) 		; year
    orbit_times(5,i) = long(strmid(buf,2,2))		; month
    orbit_times(4,i) = long(strmid(buf,4,2))		; day
    orbit_times(0,i) = long(strmid(buf,6,2))		; Hours
    orbit_times(1,i) = long(strmid(buf,8,2))		; Minutes
    orbit_times(2,i) = long(strmid(buf,10,2))		; Seconds
    free_lun,lun
endfor

if n_files gt 1 then begin
  orbit_sec = int2secarr(orbit_times,orbit_times(*,0))
  time_mid  = addtime(start_arr,delta=time_arr(6)/60.*time_arr(7)/2)

  if keyword_set(verbose) then begin
    print,'Start time = ',fmt_tim(start_arr)
    print,'Mid   time = ',fmt_tim(time_mid)
  endif

  mid_sec   = int2secarr(time_mid,orbit_times(*,0))

  xx = min(abs(mid_sec-orbit_sec)) & filid = !c & !c=0
endif else filid = 0

orbit_file = files(filid)
if keyword_set(verbose) then $
  print,'Will use orbit file: ',orbit_file,' '+fmt_tim(orbit_times(*,filid))

; --------------------------------------
;  Try to delete the output file first
; --------------------------------------
file_out = strmid(string(start_arr([6,5,4]),format='(i4.4,2i2.2)'),2,6) ; Try to guess 
stop_time = addtime(start_arr,del=time_arr(6)/60.*time_arr(7))
if keyword_set(verbose) then print,'Stop  time = ',fmt_tim(stop_time)
; Fei-Mei Lee Chou's convention for end day: less than 23:00, assign previous day
if addtime(gt_day(stop_time,/str),diff=gt_day(start_arr,/str)) ge 1440. then begin
   if addtime(stop_time,diff=gt_day(stop_time,/str)) le 23.*60 then 	$
		stop_time = addtime(stop_time,delta=(-1440.)) ; Subtract a day
endif
file_out = file_out + string('-',stop_time([5,4]),format='(a,2i2.2)')
file_out = file_out + string('_',time_arr(6),'s.ascii',format='(a,i2.2,a)')
file_out = concat_dir(path_out,file_out)
if keyword_set(verbose) then print,'Will delete file = ',file_out
file_delete,file_out,status
if status ne 1 then begin
  print,'*** Error in run_dsnfil ***' & tbeep
  print,'    Could not delete the following file = ',file_out
  print,'Delete status = ',status
  stop
endif

command = string(dsnfil_path+' ',start_arr([6,5,4,0,1,2]),		$
		time_arr(6),time_arr(7),				$
		' -p '+path_out,' -o '+orbit_file,			$
		' -r '+rigid_file,					$
		format='(a,i4,5i3,i4,i7,3a)')
if keyword_set(verbose) then print,command
spawn,command,file_out

if n_elements(file_out) ge 2 then $
     version = strtrim(file_out(1),2) else version = '???'
file_out = strtrim(file_out(0),2)
if keyword_set(verbose) then begin
   print,'DSNFIL output = ',file_out
   print,'Version = ',version
endif

end


